Attribute VB_Name = "Module1"
Option Explicit

Public Type POINTAPI
    X As Long
    Y As Long
End Type

Public Type POLY
    Corners(0 To 3) As POINTAPI
End Type

Public Type Particle
    X As Single
    Y As Single
    XOld As Single
    YOld As Single
End Type

Public Drag As Single
Public Gravity As Single
Public ColPen As Single

Public Abort As Boolean

Public NUMPARTICLES As Long
Public NUMSTICKS As Long
Public NCOLS As Long
Public NROWS As Long
Public NPOLYS As Long

Public Gap As Single
Public AverageSize As Single
Public CurtainWidth As Long
Public TopRail As Long

Public Particles() As Particle
Public StickA() As Single
Public StickB() As Single
Public Pinned() As Boolean
Public StickLen() As Single
Public StickPresent() As Boolean
Public Polygons() As POLY

Public Function Length(Point1 As Particle, Point2 As Particle) As Single

Dim DX As Single
Dim DY As Single

DX = (Point1.X - Point2.X) ^ 2
DY = (Point1.Y - Point2.Y) ^ 2

Length = Sqr(DX + DY)

End Function

Public Sub Init()

Dim Row As Long
Dim Col As Long
Dim LM As Single
Dim INDEX As Long
Dim P As Long
Dim BASE As Long
Dim PolyInd As Long

Drag = 0.999
Gravity = 0.5
ColPen = 0.9

NCOLS = 100
NROWS = 23
CurtainWidth = 600
TopRail = 20

Randomize Timer

Form1.Picture1.FillStyle = vbSolid
Form1.Picture1.ForeColor = RGB(0, 0, 0)

NUMPARTICLES = NCOLS * NROWS
NUMSTICKS = (2 * NCOLS * NROWS) - NROWS - NCOLS
NPOLYS = (NCOLS - 1) * (NROWS - 1) + (NROWS - 3)

ReDim Particles(NUMPARTICLES)
ReDim Pinned(NUMPARTICLES)

ReDim StickA(NUMSTICKS)
ReDim StickB(NUMSTICKS)
ReDim StickLen(NUMSTICKS)
ReDim StickPresent(NUMSTICKS)

ReDim Polygons(NPOLYS)

LM = (Form1.Picture1.ScaleWidth - CurtainWidth) / 2
Gap = CurtainWidth / (NCOLS - 1)
AverageSize = (Gap ^ 2) * 1.5

For Row = 0 To NROWS - 1
    For Col = 0 To NCOLS - 1
        INDEX = (Row * NCOLS) + Col
        Particles(INDEX).X = LM + (Gap * Col)
        Particles(INDEX).XOld = Particles(INDEX).X + ((Rnd * 8) - 4)
        Particles(INDEX).Y = TopRail
        Particles(INDEX).YOld = TopRail + ((Rnd * 8) - 4)
        Pinned(INDEX) = False
    Next
Next

For Row = 0 To NROWS - 1
    For Col = 0 To NCOLS - 2
        INDEX = (Row * NCOLS) + Col
        StickA(P) = INDEX
        StickB(P) = INDEX + 1
        StickLen(P) = Gap
        StickPresent(P) = True
        P = P + 1
    Next
Next

BASE = (NCOLS - 1) * NROWS
For Row = 0 To NROWS - 2
    For Col = 0 To NCOLS - 1
        INDEX = (Row * NCOLS) + Col
        StickA(INDEX + BASE) = INDEX
        StickB(INDEX + BASE) = INDEX + NCOLS
        StickLen(INDEX + BASE) = Gap
        StickPresent(INDEX + BASE) = True
    Next
Next

For Col = 0 To NCOLS - 1
    Pinned(Col) = True
Next

QueryPerformanceFrequency TicksPerSecond
QueryPerformanceCounter StartTime

Draw

End Sub

Public Sub Draw()

Dim i As Long
Dim MyPoints(0 To 1) As POINTAPI
Dim ThisSize As Single
Dim ThisPercentage As Single
Dim ThisCol As Single

Form1.Picture1.Cls

For i = 0 To NPOLYS
    ThisSize = GetArea(Polygons(i))
    ThisPercentage = ThisSize / AverageSize / 2
    ThisCol = 128 * ThisPercentage
    
    Form1.Picture1.FillColor = RGB(ThisCol, ThisCol, ThisCol)
    'Form1.Picture1.ForeColor = Form1.Picture1.FillColor
    Polygon Form1.Picture1.hdc, Polygons(i).Corners(0), 4
Next

End Sub

Public Sub UpdatePoints()

Dim i As Long
Dim VX As Single
Dim VY As Single

For i = 0 To NUMPARTICLES - 1
    If Not Pinned(i) Then
        VX = (Particles(i).X - Particles(i).XOld) * Drag
        VY = (Particles(i).Y - Particles(i).YOld) * Drag
    
        Particles(i).XOld = Particles(i).X
        Particles(i).YOld = Particles(i).Y
        
        Particles(i).X = Particles(i).X + VX
        Particles(i).Y = Particles(i).Y + VY
        
        Particles(i).Y = Particles(i).Y + Gravity
    End If
Next

For i = 0 To NPOLYS
    If (i + 1) Mod NCOLS <> 0 Then
        Polygons(i).Corners(0).X = Particles(i).X
        Polygons(i).Corners(0).Y = Particles(i).Y
        
        Polygons(i).Corners(1).X = Particles(i + 1).X
        Polygons(i).Corners(1).Y = Particles(i + 1).Y
        
        Polygons(i).Corners(2).X = Particles(i + NCOLS + 1).X
        Polygons(i).Corners(2).Y = Particles(i + NCOLS + 1).Y
        
        Polygons(i).Corners(3).X = Particles(i + NCOLS).X
        Polygons(i).Corners(3).Y = Particles(i + NCOLS).Y
    End If
Next

End Sub

Public Sub UpdateSticks()

Dim i As Long
Dim DeltaX As Single
Dim DeltaY As Single
Dim Distance As Single
Dim Diff As Single
Dim Percent As Single
Dim OffsetX As Single
Dim OffsetY As Single

For i = 0 To NUMSTICKS - 1
    If StickPresent(i) Then
        DeltaX = Particles(StickA(i)).X - Particles(StickB(i)).X
        DeltaY = Particles(StickA(i)).Y - Particles(StickB(i)).Y
        Distance = Sqr(DeltaX ^ 2 + DeltaY ^ 2)
        
        Diff = StickLen(i) - Distance
        If Diff < 0 Then
            Percent = (Diff / Distance) / 2
        
            OffsetX = DeltaX * Percent
            OffsetY = DeltaY * Percent
            
            If Not Pinned(StickA(i)) Then
                Particles(StickA(i)).X = Particles(StickA(i)).X + OffsetX
                Particles(StickA(i)).Y = Particles(StickA(i)).Y + OffsetY
            End If
            If Not Pinned(StickB(i)) Then
                Particles(StickB(i)).X = Particles(StickB(i)).X - OffsetX
                Particles(StickB(i)).Y = Particles(StickB(i)).Y - OffsetY
            End If
        End If
    End If
Next

End Sub

Public Sub ApplyBoundaries()

Dim i As Long
Dim VX As Single
Dim VY As Single

For i = 0 To NUMPARTICLES - 1
    VX = (Particles(i).X - Particles(i).XOld) * Drag
    VY = (Particles(i).Y - Particles(i).YOld) * Drag

    If Particles(i).X > Form1.Picture1.ScaleWidth Then
        Particles(i).X = Form1.Picture1.ScaleWidth
        Particles(i).XOld = Particles(i).X + (VX * ColPen)
    End If
    
    If Particles(i).X < 0 Then
        Particles(i).X = 0
        Particles(i).XOld = Particles(i).X + (VX * ColPen)
    End If
    
    If Particles(i).Y > Form1.Picture1.ScaleHeight Then
        Particles(i).Y = Form1.Picture1.ScaleHeight
        Particles(i).YOld = Particles(i).Y + (VY * ColPen)
    End If
    
    If Particles(i).Y < 0 Then
        Particles(i).Y = 0
        Particles(i).YOld = Particles(i).Y + (VY * ColPen)
    End If
Next

End Sub

Public Function GetArea(ThisPoly As POLY) As Single 'https://www.vbarchiv.net/tipps/tipp_978-polygon-zeichnen-flaechenberechnung-nach-gauss.html

Dim i As Long
Dim Area As Single
Dim Xs As Single
Dim Ys As Single
Dim Xa As Single
Dim Ya As Single

Xs = ThisPoly.Corners(0).X
Ys = ThisPoly.Corners(0).Y

For i = 0 To 3
    Xa = ThisPoly.Corners(i).X
    Ya = ThisPoly.Corners(i).Y
    
    Area = Area + (Ys + Ya) * (Xs - Xa)
    
    Xs = Xa
    Ys = Ya
Next i

Xa = ThisPoly.Corners(0).X
Ya = ThisPoly.Corners(0).Y

Area = Area + (Ys + Ya) * (Xs - Xa)

GetArea = Abs(Area) / 2
  
End Function
